import os

import boto3

from pxpip.logger import logger

log = logger(__name__)

s3 = boto3.resource(
    "s3",
    endpoint_url=os.getenv("ENDPOINT_URL", None),
    region_name=os.getenv("AWS_REGION", None),
)


def download_file(bucket, key, local_dir="./"):
    s3_object = s3.Object(bucket, key)
    file_path = os.path.join(local_dir, os.path.basename(key))
    with open(file_path, "wb") as wf:
        wf.write(s3_object.get()["Body"].read())
        log.info(f"download complete for {bucket}/{key} to {local_dir}/{key}")

    return file_path


def upload_file(bucket, key, file_path):
    """Uploads a local file system file to the provided S3 config"""
    s3.meta.client.upload_file(file_path, bucket, key)
    log.info(f"upload complete for {bucket}/{key} from {file_path}")




import logging
import os

import sys
from datetime import datetime

LOG_LEVEL = os.getenv("LOG_LEVEL", "INFO")


def logger(logger_name, log_level=LOG_LEVEL):
    logging.basicConfig()
    logger_obj = logging.getLogger(logger_name)
    logger_obj.propagate = False

    log_handler = logging.StreamHandler(sys.stdout)
    log_handler.setFormatter(LogFormatter())

    logger_obj.addHandler(log_handler)
    logger_obj.setLevel(log_level)

    return logger_obj


class LogFormatter(logging.Formatter):
    """
    Custom formatter for logging.logger instances
    """
    LOG_ATTRS = {"created", "levelname", "msg", "name"}
    DATE_TIME_FORMAT = "%Y-%m-%d %H:%M:%S.%f"

    def __init__(self):
        super(LogFormatter, self).__init__()
        self.extended = (LOG_LEVEL == "DEBUG")

    def format(self, record) -> str:
        """
        Used by logger instance to parse a log record and print an alternative string
        """

        obj = {attr: getattr(record, attr) for attr in self.LOG_ATTRS}
        obj["msg"] = obj.get("msg") % record.args if record.args else obj.get("msg")
        new_asctime = datetime.fromtimestamp(obj["created"]).strftime(self.DATE_TIME_FORMAT)[:-3]
        del obj["created"]
        return (
            obj['msg'] if not self.extended
            else
            f"{new_asctime} || {obj['name']} || {obj['levelname']} || {obj['msg']} "
        )



import os
import re
import subprocess
import sys
from abc import ABC, abstractmethod

from pxpip.common import get_bucket_and_key, get_s3_url, get_recursive_file
from pxpip.logger import logger
from pxpip.s3_facade import download_file, upload_file

log = logger(__name__)


class Command(ABC):
    def __init__(self, p_args):
        self.p_args = p_args

    @abstractmethod
    def run(self, *args, **kwargs):
        pass


class S3InstallCommand(Command):
    def __init__(self, p_args=None, s3_url=None):
        super().__init__(p_args)
        self.s3_url = get_s3_url(p_args) if not s3_url else s3_url

    def run(self):
        s3_data = get_bucket_and_key(self.s3_url)
        log.info(f"<< pxpip >> << s3 bucket install >> {s3_data}")
        local_file = download_file(**s3_data)
        pip_command = PipProxyCommand(p_args=["install", local_file])
        pip_command.run()
        os.remove(local_file)


class InstallRecursiveCommand(Command):

    def run(self, *args, **kwargs):
        if recursive_file := get_recursive_file(self.p_args):
            log.info(f"<< pxpip >> << extracting s3 >> {recursive_file}")
            lines_to_proxy = []
            s3_ran = False
            with open(recursive_file) as rf:
                for line in rf.readlines():
                    if s3_url := get_s3_url([line]):
                        install_command = S3InstallCommand(s3_url=s3_url)
                        install_command.run()
                        s3_ran = True
                    else:
                        lines_to_proxy.append(line)

            if lines_to_proxy and s3_ran:
                temp_name = "./_requirements.txt"
                with open(temp_name, 'w') as wf:
                    wf.writelines(lines_to_proxy)

                self.p_args[2] = temp_name

            pip_command = PipProxyCommand(self.p_args)
            pip_command.run()
            if lines_to_proxy and s3_ran:
                log.info("Removing temp requirements file")
                os.remove(self.p_args[2])

        else:
            raise ValueError("No requirements file was supplied.")


class S3PublishCommand(Command):
    def __init__(self, p_args):
        super().__init__(p_args)
        self.s3_url = get_s3_url(p_args)

    def run(self):
        s3_data = get_bucket_and_key(self.s3_url)
        s3_data["file_path"] = self.p_args[1]
        log.info(f"pxpip >> publishing to S3 >> {s3_data}")
        upload_file(**s3_data)


class PipProxyCommand(Command):
    def __init__(self, p_args):
        super().__init__(p_args)

    def run(self):
        process = subprocess.Popen(
            [sys.executable, "-m", "pip", *self.p_args], stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )

        for line in iter(process.stdout.readline, b''):
            print(line.decode("utf-8").strip())
        if exitcode := process.wait():  # 0 means success
            raise ValueError(f"Pip could not process the passed arguments {self.p_args}. exit code : {exitcode}")


class CommandFactory:
    S3_INSTALL_REGEX = re.compile(r'install s3://([^/]+)/(.*?([^/]+)/?)')
    S3_PUBLISH_REGEX = re.compile(r'publish [a-zA-Z0-9./_-]+ s3://([^/]+)/(.*?([^/]+)/?)')
    RECURSIVE_INSTALL = re.compile(r'install -r ([a-zA-Z]+)(.*?([^/]+)/?)')

    @classmethod
    def get_command_class(cls, p_args: list[str]):
        p_str = " ".join(p_args)
        if cls.S3_INSTALL_REGEX.match(p_str):
            return S3InstallCommand(p_args)
        elif cls.RECURSIVE_INSTALL.match(p_str):
            return InstallRecursiveCommand(p_args)
        elif cls.S3_PUBLISH_REGEX.match(p_str):
            return S3PublishCommand(p_args)
        else:
            return PipProxyCommand(p_args)



def get_bucket_and_key(s3_url):
    s3_url = s3_url.split("/") if s3_url else s3_url
    return {
        "bucket": s3_url[2],
        "key": '/'.join(s3_url[3:])
    } if s3_url else None


def get_s3_url(p_args):
    for arg in p_args:
        if 's3://' in arg:
            return arg


def get_recursive_file(p_args):
    return p_args[2]



from setuptools import setup, find_packages

__version__ = "0.0.2"

with open("README.md") as f:
    readme = f.read()

setup(
    name="pxpip",
    version=__version__,
    description="A Proxy wrapper around pip to enable cloud interfaces without the need for pipy",
    long_description=readme,
    author="claytonbezuidenhout",
    url="https://github.com/claytonbezuidenhout/pxpip",
    packages=find_packages(exclude=("tests", "dist", "docker", "venv")),
    install_requires=[
        "boto3==1.20.26"
    ],
    entry_points={
        'console_scripts': [
            'pxpip = pxpip:main'
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
    ],
    python_requires=">=3.8",
)
